<?php
/**
 * Pipelines SPIP utilisés par le plugin
 *
 * @author		Jonathan OCHEJ
 * @license		MIT - https://github.com/GouvernementFR/dsfr/blob/main/LICENSE.md
 * 
 **/
if ( !defined('_ECRIRE_INC_VERSION') ) {
	return;
}

/**
 * Déclarer les interfaces des tables SPIP
 *
 * @pipeline declarer_tables_interfaces
 *
 * @param array $interface
 * 		Déclarations d'interface pour le compilateur
 * @return array $interface
 * 		Déclarations d'interface complétée pour le compilateur
 **/
function dsfr_composants_declarer_tables_interfaces($interface) {

	if ( !defined('_DSFR_TRAITEMENT_TRI') ) {
		define('_DSFR_TRAITEMENT_TRI', 'dsfr_traitement_tri(%s, $connect, $Pile[0])');
	}

	// Permet de définir des traitements systématiques (filtres) sur certaines
	// balises SPIP. L'étoile (#BALISE*) désactive ces traitements.
	$interface['table_des_traitements']['TRI'][] = _DSFR_TRAITEMENT_TRI;

	return $interface;
}

/**
 * Ajoute le bouton d'administration indiquant la version du DSFR utilisé
 * 
 * @pipeline formulaire_admin
 * 
 * @param array $flux
 * 
 * @return array $flux
 **/
function dsfr_composants_formulaire_admin($flux) {

	// ne pas ajouter les boutons d'administration lors d'une erreur 503
	if ( http_response_code() == 503 ) {
		return false;
	}

	$bouton = recuperer_fond('prive/bouton/dsfr_version');
	if ( !empty($bouton) ) {
		$extra = '<!--extra-->';
		$flux['data'] = str_ireplace($extra, "\n\t".$bouton."\n\t".$extra, $flux['data']);
	}

	return $flux;
}

/**
 * Insérer favicon et manifest
 * 
 * @pipeline insert_head
 * 
 * @param string $flux
 *     Le contenu de la balise #INSERT_HEAD
 * 
 * @return string
 **/
function dsfr_composants_insert_head($flux) {

	$flux .= "\n";
	$flux .= "\t".'<!-- DSFR -->'."\n";

	// ajoute les meta
	$flux .= "\t".'<meta charset="'.lire_config('charset').'">'."\n";
	$flux .= "\t".'<meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">'."\n";
	$flux .= "\t".'<meta name="format-detection" content="telephone=no,date=no,address=no,email=no,url=no">'."\n";
	$flux .= "\t".'<meta name="theme-color" content="#000091">'."\n";

	// ajoute le favicon
	$flux .= "\t".'<link rel="apple-touch-icon" href="'.timestamp(dsfr('favicon/apple-touch-icon.png')).'">'."\n";
	$flux .= "\t".'<link rel="icon" href="'.timestamp(dsfr('favicon/favicon.svg')).'" type="image/svg+xml">'."\n";
	$flux .= "\t".'<link rel="shortcut icon" href="'.timestamp(dsfr('favicon/favicon.ico')).'" type="image/x-icon">'."\n";
	$flux .= "\t".'<link rel="manifest" href="'.timestamp(dsfr('favicon/manifest.webmanifest')).'" crossorigin="use-credentials">'."\n";

	$verbose = 'false';
	$mode = 'auto';
	$analytics = '';

	if ( dsfr_composants_configuration('activer_options_api_js') == 'oui' ) {
		if ( dsfr_composants_configuration('api_js_option_verbose') == 'oui' ) {
			$verbose = 'true';
		}
		$mode = dsfr_composants_configuration('api_js_option_mode');
	}

	if ( dsfr_composants_configuration('charger_api_analytics') == 'oui' ) {
		$analytics =  trim(dsfr_composants_configuration('api_analytics_configuration'));
		if ( !empty($analytics) ) {
			$analytics = ','."\n\t\t\t".'analytics: { '.$analytics.'}';
		}
	}

	// ajoute la configuration de l'API du DSFR
	$configuration_DSFR = <<<HEREDOC_JAVASCRIPT
		<script>
			window.dsfr = {
				verbose: $verbose,
				mode: '$mode'$analytics
			}
		</script>
	HEREDOC_JAVASCRIPT;

	$flux .= $configuration_DSFR;

	return $flux;
}

/**
 * Insérer feuilles de styles
 * 
 * @pipeline insert_head_css
 * 
 * @param string $flux
 *     Le contenu de la balise #INSERT_HEAD_CSS
 * 
 * @return string
 **/
function dsfr_composants_insert_head_css($flux) {

	// ajoute les CSS
	$flux .= "\n";
	$flux .= "\t".'<!-- DSFR:css -->'."\n";
	$flux .= "\t".'<link rel="stylesheet" href="'.timestamp(dsfr('dsfr.min.css')).'">'."\n";
	if ( dsfr_composants_configuration('charger_css_utility') == 'oui' ) {
		$flux .= "\t".'<link rel="stylesheet" href="'.timestamp(dsfr('utility/utility.min.css')).'">'."\n";
	}
	// si il y a des icônes DSFR perso
	if ( dsfr_liste_icones('type=perso') ) {
		$flux .= "\t".'<!-- DSFR icônes -->'."\n";
		$flux .= "\t".'<link rel="stylesheet" href="'.produire_fond_statique('css/dsfr_icones.css').'">'."\n";
	}
	$flux .= "\t".'<!-- DSFR composants -->'."\n";
	$flux .= "\t".'<link rel="stylesheet" href="'.produire_fond_statique('css/dsfr_composants.css').'">'."\n";

	return $flux;
}

/**
 * Insérer javascript et data-fr-scheme="system"
 * 
 * @pipeline affichage_final
 * 
 * @param string $flux
 * 
 * @return string
 **/
function dsfr_composants_affichage_final($flux) {

	// ne rien afficher
	if ( !isset($GLOBALS['html']) || !$GLOBALS['html'] ) {
		return $flux;
	}

	// ne rien faire lors d'une erreur 503
	if ( http_response_code() == 503 ) {
		return $flux;
	}

	// nécessaire pour `timestamp()` et `produire_fond_statique()`
	include_spip('inc/filtres');

	// ajoute data-fr-scheme="system" à la balise <html>
	$flux = preg_replace('/<html(.*)>/i','<html$1 data-fr-scheme="system">', $flux);

	// ajoute les fichiers javascript du DSFR.
	// le commentaire `<!-- DSFR:javascript -->` très important et doit être conservé !
	// il permet d'ajouter du contenu supplementaire juste avant le chargement de l'ensemble
	// du code javascript nécessaire au bon fonctionnement du DSFR (utilisé par le plugin `DSFR Squelettes`).
	$ajouter  = "\n".'<!-- DSFR:javascript -->'."\n";
	$ajouter .= '<script type="module" src="'.timestamp(dsfr('dsfr.module.min.js')).'"></script>'."\n";
	$ajouter .= '<script type="text/javascript" nomodule src="'.timestamp(dsfr('dsfr.nomodule.min.js')).'"></script>'."\n";

	// API analytics du DSFR
	if ( dsfr_composants_configuration('charger_api_analytics') == 'oui' ) {
		$analytics =  trim(dsfr_composants_configuration('api_analytics_configuration'));
		if ( !empty($analytics) ) {
			$ajouter .= '<!-- DSFR analytics -->'."\n";
			$ajouter .= '<script type="module" src="'.timestamp(dsfr('analytics/analytics.module.min.js')).'"></script>'."\n";
			$ajouter .= '<script type="text/javascript" nomodule src="'.timestamp(dsfr('analytics/analytics.nomodule.min.js')).'"></script>'."\n";
		}
	}

	// javascript spécifique au plugin
	$ajouter .= '<!-- DSFR composants -->'."\n";
	$ajouter .= '<script type="text/javascript" src="'.produire_fond_statique('javascript/dsfr_composants.js').'"></script>'."\n";

	// ajoute le contenu juste avant la fermeture de la balise `<body>`
	$flux = str_replace('</body>', $ajouter.'</body>', $flux);

	return $flux;
}

/**
 * @pipeline affichage_final_prive
 * 
 * @param string $flux
 * 
 * @return string
 **/
function dsfr_composants_affichage_final_prive($flux) {

	// uniquement pour la configuration/documentation du plugin
	if ( _request('exec') == 'configurer_dsfr_composants' || _request('exec') == 'documentation_dsfr_composants') {
		/*
			prismjs.com : librairie javascript de colorisation syntaxique.
			Cette fonctionnalitée permettant uniquement un confort d'affichage, j'ai choisi d'utiliser un CDN pour ne par surcharger le contenu du plugin.
			Ce n'est PAS GRAVE si la librairie ne se charge pas, la colorisation syntaxique n'étant PAS un prérequis necessaire au bon fonctionnement du plugin.
		*/
		$ajouter  = "\n".'<!-- prismjs.com -->'."\n";
		$ajouter .= '<link rel="stylesheet" href="https://unpkg.com/prismjs@1.29.0/themes/prism.min.css">'."\n";
		$ajouter .= '<script type="text/javascript" src="https://unpkg.com/prismjs@1.29.0/components/prism-core.min.js"></script>'."\n";
		$ajouter .= '<script type="text/javascript" src="https://unpkg.com/prismjs@1.29.0/plugins/autoloader/prism-autoloader.min.js"></script>'."\n";
	
		// ajoute le contenu juste avant la fermeture de la balise `<body>`
		$flux = str_replace('</body>', $ajouter.'</body>', $flux);
	}

	return $flux;
}

/**
 * Désactive la mediabox sur le site public pour ne pas interférer avec le DSFR
 * 
 * @pipeline mediabox_config
 * 
 * @param array $config
 * 
 * @return array
 **/
function dsfr_composants_mediabox_config($config) {
	// uniquement sur le site public
	if ( !test_espace_prive() ) {
		$config['active'] = 'non';
	}

	return $config;
}

/**
 * Ajoute les aides du compagnon
 * 
 * @pipeline compagnon_messages
 * 
 * @param array $flux
 * 
 * @return array
 **/
function dsfr_composants_compagnon_messages($flux) {

	// uniquement sur l'espace privé
	if ( test_espace_prive() ) {

		$exec = $flux['args']['exec'];
		$pipeline = $flux['args']['pipeline'];
		$aides = &$flux['data'];

		switch ( $pipeline ) {
			case 'affiche_milieu':
				switch ( $exec ) {
					case 'configurer_dsfr_composants':
					case 'documentation_dsfr_composants':
						$aides[] = [
							'id' => 'dsfr_composants',
							'titre' => <<<HEREDOC_TITRE
								Qu'est-ce que le Système de Design de l'État ?
								HEREDOC_TITRE,
							'texte' => <<<HEREDOC_TEXTE
								<p>
									Ce plugin fait partit du volet numérique de la marque de l'État, qui permet pour les citoyens d'avoir une cohérence 
									graphique et une meilleure expérience à travers l'ensemble des sites de l'État.
								</p>
								<p>
									Le Système de Design de l'État regroupe un ensemble de composants réutilisables, 
									répondant à des standards et à une gouvernance, pouvant être assemblés pour créer
									des sites Internet accessibles et ergonomiques.
								</p>
								<p>
									<a href="https://www.systeme-de-design.gouv.fr" target="_blank" class="btn">Prendre en main le Système de Design</a>
								</p>
							HEREDOC_TEXTE,
							'statuts' => ['webmestre']
						];
						break;
				}
				break;
		}

	}

	return $flux;
}

/**
 * Ajoute les metas sauvegardables pour le plugin IEConfig
 *
 * @pipeline ieconfig_metas
 *
 * @param array $table
 * 		Déclaration des sauvegardes
 * @return array
 * 		Déclaration des sauvegardes complétées
 **/
function dsfr_composants_ieconfig_metas($table) {
	$table['dsfr_composants_meta']['titre'] = _T('dsfr_composants:dsfr_composants');
	$table['dsfr_composants_meta']['icone'] = 'dsfr_composants.svg';
	$table['dsfr_composants_meta']['metas_serialize'] = 'dsfr_composants';

	return $table;
}